#uses "fwUnitTestComponentAsserts.ctl"
#uses "CtrlPv2Admin"

#uses "test/unFrontEnd/unFrontEndTest_helper.ctl"

#uses "unConfigGenericFunctions.ctl"

const string DEVICES_DRIVER_NUMBER = "3";

//this is the name of FrontEnd imported by setupSuite()
const string FRONTENDDPTEST = "_unPlc_PLCENICE03.configuration.type:_original_value"; //gives a DPE instead of a DP on purpose to check that the name is properly parsed
const string CONFIGFILEPATH = getPath(CONFIG_REL_PATH, "config");
const string MODBUS_SECTION = "mod_"+DEVICES_DRIVER_NUMBER;

const string NEW_PORT     ="5210";
const string DEFAULT_PORT ="502";


//-------------------------------------------------------
void test_unFrontEnd_deleteSubApplication_setupSuite() {

  //config cleanup in case another test did not clean its entries
  int rc=paCfgDeleteValue(CONFIGFILEPATH,MODBUS_SECTION,"tcpServerPort");
  
	unImportTestDevice_helper_startDriver(DEVICES_DRIVER_NUMBER);
	fwDevice_initialize(); //needed by unImportDevice_checkFile()... dont ask me why

	_importFrontEnd();
  
  _checkTcpPortServerConfigEntry(DEFAULT_PORT, "Initialization");
}

//-------------------------------------------------------
void test_unFrontEnd_deleteSubApplication_teardownSuite() {
	dyn_string dsDp=dpNames("*PLCENICE03*");
	dynAppend( dsDp, dpNames("_Mod_Plc_3"));
	for(int i=1; i<=dynlen(dsDp);i++)	dpDelete(dsDp[i]);
  
  //and cleanup port file
  int rc=paCfgDeleteValue(CONFIGFILEPATH,MODBUS_SECTION,"tcpServerPort");
}

//-----------------------------------------------------------
//c.f. UNTOOL-53 
void test_unFront_notDuplicatedTcpserverportEntryAfterReimport(){
  //simluate port redirection
  
  int rc=paCfgDeleteValue(CONFIGFILEPATH,MODBUS_SECTION,"tcpServerPort");
  int rc=paCfgInsertValue(CONFIGFILEPATH,MODBUS_SECTION,"tcpServerPort",NEW_PORT);
  _checkTcpPortServerConfigEntry(NEW_PORT, "Modbus redirect to "+NEW_PORT);
  
  //import another time the same frontend and check that the tcpServerPort stays the same
  _importFrontEnd();
  
  //and check that the tcpPortServer did not change and is still unique
  _checkTcpPortServerConfigEntry(NEW_PORT, "FE re-import.");
}


private void _checkTcpPortServerConfigEntry( string expectedValue, string stepDesc){
  //check first that there is only one entry of tcpServerPort
  dyn_string valueList;
  int rc=paCfgReadValueList(CONFIGFILEPATH,MODBUS_SECTION,"tcpServerPort",valueList);
  
  assertEqual( 1, dynlen(valueList), "The number of tcpServerPortEntry config is not one in step "+stepDesc);
  assertEqual( expectedValue, valueList[1],  "Incorrect port number retrieved from configuration file in step: "+stepDesc);
}

private void _importFrontEnd(){
  //Then import  applications of the same frontend
	string sLogContent;
	string sImportFile1 = "test_unFrontEnd/wincc_oa_db_file_Schneider_PLCENICE03_M580.txt";
  bool importOk = unImportTestDevice_helper_importDevices(sImportFile1, DEVICES_DRIVER_NUMBER, sLogContent);
	assertEqual( true, importOk, "Unable to import front end "+sImportFile1+" "+sLogContent);
}

