#uses "fwUnitTestComponentAsserts.ctl"
#uses "unSystemIntegrity/unSendMail.ctl"
#uses "fwConfigs/fwAlertConfig.ctl"

/**
 * Unit test functions following UNSYSINT-45 to fully test the function
 * void _unSendMail_Status_Value(string sDpe, string &sVal, string &sAl, string &sInfo)
 **/

const string CPC_DEVICE_TEST1="cpc-test-1";
const string CPC_DEVICE_TEST2="cpc-test-2";
const string CPC_DEVICE_TEST3="cpc-test-3";
const string CPC_POSST_SUFFIX=".ProcessInput.PosSt";

//-------------------------------------
void test_unSendMail_setupSuite(){
  //no need to check for error code, as the correct execution is checked by the assertEqual
  dpCreate(CPC_DEVICE_TEST1, "CPC_DigitalInput");
  dpCreate(CPC_DEVICE_TEST2, "CPC_DigitalInput");
  dpCreate(CPC_DEVICE_TEST3, "CPC_DigitalInput");
  _setAlertHandler(CPC_DEVICE_TEST2+CPC_POSST_SUFFIX, true);
  _setAlertHandler(CPC_DEVICE_TEST3+CPC_POSST_SUFFIX, false);
  assertEqual( true, dpExists(CPC_DEVICE_TEST1), "DP "+CPC_DEVICE_TEST1+" could not be created");
  assertEqual( true, dpExists(CPC_DEVICE_TEST2), "DP "+CPC_DEVICE_TEST2+" could not be created");
  assertEqual( true, dpExists(CPC_DEVICE_TEST3), "DP "+CPC_DEVICE_TEST2+" could not be created");
}

//-------------------------------------
void test_unSendMail_teardownSuite(){
  //no need to check for error code, as the correct execution is checked by the assertEqual
  dpDelete(CPC_DEVICE_TEST1);
  dpDelete(CPC_DEVICE_TEST2);
  dpDelete(CPC_DEVICE_TEST3);
  assertEqual( false, dpExists(CPC_DEVICE_TEST1), "DP "+CPC_DEVICE_TEST1+" still exists while it should have been deleted");
  assertEqual( false, dpExists(CPC_DEVICE_TEST2), "DP "+CPC_DEVICE_TEST2+" still exists while it should have been deleted");
  assertEqual( false, dpExists(CPC_DEVICE_TEST3), "DP "+CPC_DEVICE_TEST2+" still exists while it should have been deleted");
 }

//-------------------------------------
void test_getStatusValue_NoAlarm(){
  string sDpe = CPC_DEVICE_TEST1+CPC_POSST_SUFFIX;
  string sVal; 
  string sAl; 
  string sInfo;
  _unSendMail_Status_Value( sDpe, sVal, sAl, sInfo);
  assertEqual("FALSE", sVal, "The value is not correct");
  assertEqual("NONE", sAl, "The alarm configuration is not correctly detected");
  assertEqual("         -", sInfo, "The info text is not correct");
}

//-------------------------------------
void test_getStatusValue_AlarmEnabled(){
  string sDpe = CPC_DEVICE_TEST2+CPC_POSST_SUFFIX;
  string sVal; 
  string sAl; 
  string sInfo;
  _unSendMail_Status_Value( sDpe, sVal, sAl, sInfo);
  assertEqual("FALSE", sVal, "The value is not correct");
  assertEqual("BAD", sAl, "The alarm configuration is not correctly detected");
  assertEqual("Need Ack -", sInfo, "The info text is not correct");
}

//-------------------------------------
void test_getStatusValue_AlarmDisabled(){
  string sDpe = CPC_DEVICE_TEST3+CPC_POSST_SUFFIX;
  string sVal; 
  string sAl; 
  string sInfo;
  _unSendMail_Status_Value( sDpe, sVal, sAl, sInfo);
  assertEqual("FALSE", sVal, "The value is not correct");
  assertEqual("BAD", sAl, "The alarm configuration is not correctly detected");
  assertEqual("Disabled -", sInfo, "The info text is not correct");
}


//-------------------------------------
private void _setAlertHandler( string sDpe, bool bActive){
  dyn_string dsException;
  dyn_mixed alertConfigObject;
  fwAlertConfig_objectCreateDigital(alertConfigObject, 
                                    makeDynString("BAD","OK"),                                    
                                    makeDynString("_cpcAnalog_Ack.", ""),
                                    "",
                                    makeDynString(),
                                    "",                                    
                                    dsException
                                  );
  assertEmpty(dsException, "Error while creating alert config object for "+sDpe+": "+dsException);

  fwAlertConfig_objectSet( sDpe, alertConfigObject, dsException);
  assertEmpty(dsException, "Error while setting alert config object on "+sDpe+": "+dsException);

  if ( bActive==true){
    fwAlertConfig_activate(sDpe, dsException);
    assertEmpty(dsException, "Error activating alert config on "+sDpe+": "+dsException);
  }
}

